/*
 *  gauge.c -- implements the gauge dialog
 *
 *  Originally by Marc Ewing, Red Hat Software
 *  Completely reworked by Alessandro Rubini (rubini@ipvvis.unipv.it).
 *  Reworked yet again by Donnie Barnes (djb@redhat.com), Red Hat Software
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "dialog.h"
#include "dialogInt.h"

/*
 * Display a gauge, or progress meter.  Starts at percent% and
 * reads stdin.  If stdin is not XXX, then it is interpreted as
 * a percentage, and the display is updated accordingly.  Otherwise
 * the next line is the percentage, and subsequent lines up to
 * another XXX are used for the new prompt.  Note that the size
 * of the window never changes, so the prompt can not get any
 * larger than the height and width specified.
 */

int
dialog_gauge (Dialog_desc *desc)
{
    char *prompt;
    int percent;
    char *buf, *prompt_buf;

    if ( (prompt_buf=malloc(1024))==NULL) return DIALOG_ERROR;
    if ( (buf=malloc(1024))==NULL) {free(prompt_buf); return DIALOG_ERROR;}

    /* first: build the window with the initial prompt/percent */
    dialog_gauge_update(desc,NULL,-1);

    while(1) {
	percent=-1;
	prompt=NULL;

	if (!gets (buf)) break;

	if (buf[0] == 'X') {
	    /* Next line is percentage */
	    gets (buf);
	    percent = atoi (buf);

	    /* Rest is message text */
	    prompt_buf[0] = '\0';
	    while (strncmp (gets (buf), "XXX", 3))
		strcat (prompt_buf, buf);
	    prompt = prompt_buf;
	} else
	  percent = atoi (buf);

	dialog_gauge_update(desc,prompt,percent);
    }

    delwin (desc->win);
    free(buf);
    free(prompt_buf);
    return (0);
}

/*
 * This function updates the gauge.  You need to pass it a pointer to
 * an existing gauge struct.  You can then pass a string to update the
 * prompt, or you can pass NULL to leave the prompt alone.  You can also
 * pass a new percentage to update with, or you can pass -1 to leave the
 * percentage alone.
 *
 * If desc->win is NULL, this initializes the gauge as well.
 */

int
dialog_gauge_update(Dialog_desc *desc, char *prompt, int percent)
{
    int i, x;
    WINDOW *dialog;
    char *buf=malloc(128);

    /*
     * If the prompt string is null, leave the desc->prompt alone.  Otherwise
     * set it.
     */
    if (prompt != NULL) desc->prompt = prompt;

    if (desc->win == NULL) { /* first invocation: build the frame */
	
	_dialog_setup(desc); /* no lines, no buttons */

	/* the box for the percentage */
	_dialog_draw_box (desc->win, desc->hei - 4, 3, 3, desc->wid - 6,
			  dialog_attr, border_attr);

    } else if (prompt != NULL) {  /* not first invocation: redraw the prompt */
	int i,j;
	for (i=1; i < desc->hei-4; i++) { /* clear the field, first */
	    wmove(desc->win,i,1);
	    for (j=1; j < desc->wid-1; j++) {
		waddch(desc->win,' ');
	    }
	}
	_dialog_print_autowrap (desc->win, desc->prompt, desc->wid - 2, 1, 3);
    }

    dialog = desc->win;

    wattrset (dialog, title_attr);
    wmove (dialog, desc->hei - 3, (desc->wid / 2) - 2);
 
    /*
     * If you percent is greater than or equal to 0, update the percent bar.
     * desc->xtra is used as a place holder for the current percentage.
     */
    if (percent >= 0) {
	desc->xtra=percent;

        sprintf (buf, "%3d%%", percent);
        waddstr (dialog, buf);

        x = (percent * (desc->wid - 8)) / 100;
        wattrset (dialog, item_selected_attr);
        wmove (dialog, desc->hei - 3, 4);
        for (i = 0; i < x; i++)
          waddch (dialog, winch (dialog));
        desc->xtra = percent;
#if 0
    /*
     * If percent was less than 0 (should be -1) then just redraw the percent
     * bar using the placeholder (desc->xtra).
     */
    } else {
        percent = desc->xtra;
        sprintf (buf, "%3d%%", percent);
        waddstr (dialog, buf);

        x = (percent * (desc->wid - 8)) / 100;
        wattrset (dialog, item_selected_attr);
        wmove (dialog, desc->hei - 3, 4);
        for (i = 0; i < x; i++)
          waddch (dialog, winch (dialog));
        desc->xtra = percent;
#endif
    }

    wrefresh(dialog);
    free(buf);
    return 0; /* succeed */
}

int
dialog_gauge_end (Dialog_desc *desc)
{
    delwin(desc->win);
    refresh();
    desc->win=NULL; /* better safe than sorry */
    return 0;
}



/**** Local Variables:  ****/
/**** tab-width:8       ****/
/**** c-indent-level:4  ****/
/**** c-label-offset:-2 ****/
/**** End:              ****/

